<?php

namespace spec\BitBag\SyliusBonusPointsPlugin\Calculator;

use BitBag\SyliusBonusPointsPlugin\Calculator\BonusPointsStrategyCalculatorInterface;
use BitBag\SyliusBonusPointsPlugin\Calculator\DelegatingBonusPointsStrategyCalculatorInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsStrategyInterface;
use Mockery\Matcher\Any;
use PhpSpec\ObjectBehavior;
use Sylius\Component\Registry\ServiceRegistryInterface;

class DelegatingBonusPointsStrategyCalculatorSpec extends ObjectBehavior
{
    function it_is_a_bonus_points_strategy_calculator()
    {
        $this->shouldImplement(DelegatingBonusPointsStrategyCalculatorInterface::class);
    }

    function let(
        ServiceRegistryInterface $registry,
        BonusPointsStrategyCalculatorInterface $calculator
    )
    {
        $registry->get('default')->willReturn($calculator);

        $this->beConstructedWith($registry);
    }

    function it_should_delegate_calculation_to_a_correct_calculator(
        BonusPointsStrategyInterface $bonusPointsStrategy,
        BonusPointsStrategyCalculatorInterface $calculator,
        Any $subject
    )
    {
        $calculatorConfiguration = ['foo' => 'bar'];

        $bonusPointsStrategy
            ->getCalculatorType()
            ->willReturn('default');
        $bonusPointsStrategy
            ->getCalculatorConfiguration()
            ->willReturn($calculatorConfiguration);

        $calculator
            ->calculate($subject, $calculatorConfiguration, 158)
            ->willReturn(1750)
            ->shouldBeCalledOnce();

        $this
            ->calculate($subject, $bonusPointsStrategy, 158)
            ->shouldReturn(1750);
    }

    function it_should_return_the_type_of_the_calculator(
        BonusPointsStrategyInterface $bonusPointsStrategy,
        BonusPointsStrategyCalculatorInterface $calculator
    )
    {
        $bonusPointsStrategy
            ->getCalculatorType()
            ->willReturn('default');

        $calculator
            ->isPerOrderItem()
            ->willReturn(true)
            ->shouldBeCalledOnce();

        $this
            ->isPerOrderItem($bonusPointsStrategy)
            ->shouldReturn(true);
    }
}
