<?php

namespace spec\BitBag\SyliusBonusPointsPlugin\Calculator;

use BitBag\SyliusBonusPointsPlugin\Calculator\BonusPointsStrategyCalculatorInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsStrategy;
use Mockery\Matcher\Any;
use PhpSpec\ObjectBehavior;
use Sylius\Component\Channel\Context\ChannelContextInterface;
use Sylius\Component\Core\Exception\MissingChannelConfigurationException;
use Sylius\Component\Core\Model\ChannelInterface;
use Sylius\Component\Core\Model\OrderInterface;

class FixedAmountCalculatorSpec extends ObjectBehavior
{
    function it_is_a_bonus_points_strategy_calculator()
    {
        $this->shouldImplement(BonusPointsStrategyCalculatorInterface::class);
    }

    function let(ChannelContextInterface $channelContext)
    {
        $this->beConstructedWith($channelContext);
    }

    function it_is_not_per_order_item()
    {
        $this->isPerOrderItem()->shouldReturn(false);
    }

    function it_returns_fixed_amount_type()
    {
        $this->getType()->shouldReturn('fixed_amount');
    }

    function it_is_compatible_with_order_and_event_strategies()
    {
        $this->getCompatibleStrategyTypes()->shouldContain(BonusPointsStrategy::TYPE_ORDER);
        $this->getCompatibleStrategyTypes()->shouldContain(BonusPointsStrategy::TYPE_EVENT);
    }

    function it_returns_the_amount_configured_on_the_channel(
        OrderInterface $order,
        ChannelInterface $channel
    )
    {
        $order->getChannel()->willReturn($channel);
        $channel->getCode()->willReturn('fr_FR');

        $this->calculate($order, ['fr_FR' => ['amount' => 100], 'en_US' => ['amount' => 333]])->shouldReturn(100);
    }

    function it_throws_a_channel_not_defined_exception_if_channel_code_key_does_not_exist(
        OrderInterface $order,
        ChannelInterface $channel
    )
    {
        $order->getChannel()->willReturn($channel);
        $channel->getCode()->willReturn('fr_FR');
        $channel->getName()->willReturn('default');

        $this
            ->shouldThrow(MissingChannelConfigurationException::class)
            ->during('calculate', [$order, []])
        ;
    }

    function it_uses_the_context_channel_if_it_is_not_and_order(
        Any $subject,
        ChannelContextInterface $channelContext,
        ChannelInterface $channel
    )
    {
        $channelContext->getChannel()->willReturn($channel);
        $channel->getCode()->willReturn('en_US');

        $this->calculate($subject, ['fr_FR' => ['amount' => 100], 'en_US' => ['amount' => 333]])->shouldReturn(333);
    }
}
