<?php

namespace spec\BitBag\SyliusBonusPointsPlugin\Calculator;

use BitBag\SyliusBonusPointsPlugin\Calculator\BonusPointsStrategyCalculatorInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsStrategy;
use InvalidArgumentException;
use Mockery\Matcher\Any;
use PhpSpec\ObjectBehavior;
use Sylius\Component\Core\Exception\MissingChannelConfigurationException;
use Sylius\Component\Core\Model\ChannelInterface;
use Sylius\Component\Core\Model\OrderInterface;

class OrderTotalPercentageCalculatorSpec extends ObjectBehavior
{
    function it_is_a_bonus_points_strategy_calculator()
    {
        $this->shouldImplement(BonusPointsStrategyCalculatorInterface::class);
    }

    function it_is_not_per_order_item()
    {
        $this->isPerOrderItem()->shouldReturn(false);
    }

    function it_returns_order_total_percentage_type()
    {
        $this->getType()->shouldReturn('order_total_percentage');
    }

    function it_is_compatible_with_order_strategies()
    {
        $this->getCompatibleStrategyTypes()->shouldContain(BonusPointsStrategy::TYPE_ORDER);
    }

    function it_is_not_compatible_with_event_strategies()
    {
        $this->getCompatibleStrategyTypes()->shouldNotContain(BonusPointsStrategy::TYPE_EVENT);
    }

    function it_returns_the_floored_percent_of_the_order_total_configured_on_the_channel(
        OrderInterface $order,
        ChannelInterface $channel
    )
    {
        $order->getChannel()->willReturn($channel);
        $order->getTotal()->willReturn(1654);
        $channel->getCode()->willReturn('fr_FR');

        $this->calculate($order, ['fr_FR' => ['percentToCalculatePoints' => 20]])->shouldReturn(330);
    }

    function it_deduces_the_amount_from_the_total_used_to_calculate_the_points(
        OrderInterface $order,
        ChannelInterface $channel
    )
    {
        $order->getChannel()->willReturn($channel);
        $order->getTotal()->willReturn(1654);
        $channel->getCode()->willReturn('fr_FR');

        $this->calculate($order, ['fr_FR' => ['percentToCalculatePoints' => 20]], -333)->shouldReturn(264);
    }

    function it_thows_and_invalid_argument_exceptions_if_the_subject_is_not_and_order(Any $subject)
    {
        $this
            ->shouldThrow(InvalidArgumentException::class)
            ->during('calculate', [$subject, []]);
    }

    function it_throws_a_channel_not_defined_exception_if_channel_code_key_does_not_exist(
        OrderInterface $order,
        ChannelInterface $channel
    )
    {
        $order->getChannel()->willReturn($channel);
        $channel->getCode()->willReturn('fr_FR');
        $channel->getName()->willReturn('default');

        $this
            ->shouldThrow(MissingChannelConfigurationException::class)
            ->during('calculate', [$order, []]);
    }
}
