<?php

namespace spec\BitBag\SyliusBonusPointsPlugin\Calculator;

use BitBag\SyliusBonusPointsPlugin\Calculator\BonusPointsStrategyCalculatorInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsStrategy;
use InvalidArgumentException;
use Mockery\Matcher\Any;
use PhpSpec\ObjectBehavior;
use Sylius\Component\Core\Exception\MissingChannelConfigurationException;
use Sylius\Component\Core\Model\ChannelInterface;
use Sylius\Component\Core\Model\Order;
use Sylius\Component\Core\Model\OrderInterface;
use Sylius\Component\Core\Model\OrderItemInterface;

class PerOrderItemPercentageCalculatorSpec extends ObjectBehavior
{
    function it_is_a_bonus_points_strategy_calculator()
    {
        $this->shouldImplement(BonusPointsStrategyCalculatorInterface::class);
    }

    function it_is_per_order_item()
    {
        $this->isPerOrderItem()->shouldReturn(true);
    }

    function it_returns_per_order_item_percentage_type()
    {
        $this->getType()->shouldReturn('per_order_item_percentage');
    }

    function it_is_compatible_with_order_strategies()
    {
        $this->getCompatibleStrategyTypes()->shouldContain(BonusPointsStrategy::TYPE_ORDER);
    }

    function it_is_not_compatible_with_event_strategies()
    {
        $this->getCompatibleStrategyTypes()->shouldNotContain(BonusPointsStrategy::TYPE_EVENT);
    }

    function it_returns_the_rounded_percent_of_the_order_item_total_configured_on_the_channel(
        Order $order,
        OrderItemInterface $orderItem,
        ChannelInterface $channel
    )
    {
        $orderItem->getOrder()->willReturn($order);
        $orderItem->getTotal()->willReturn(361);
        $order->getChannel()->willReturn($channel);
        $channel->getCode()->willReturn('fr_FR');

        $this->calculate($orderItem, ['fr_FR' => ['percentToCalculatePoints' => 20]])->shouldReturn(72);
    }

    function it_deduces_the_amount_from_the_total_used_to_calculate_the_points(
        Order $order,
        OrderItemInterface $orderItem,
        ChannelInterface $channel
    )
    {
        $orderItem->getOrder()->willReturn($order);
        $orderItem->getTotal()->willReturn(361);
        $order->getChannel()->willReturn($channel);
        $channel->getCode()->willReturn('fr_FR');

        $this->calculate($orderItem, ['fr_FR' => ['percentToCalculatePoints' => 20]], -70)->shouldReturn(58);
    }

    function it_thows_and_invalid_argument_exceptions_if_the_subject_is_not_and_order_item(Any $subject)
    {
        $this
            ->shouldThrow(InvalidArgumentException::class)
            ->during('calculate', [$subject, []]);
    }

    function it_throws_a_channel_not_defined_exception_if_channel_code_key_does_not_exist(
        Order $order,
        OrderItemInterface $orderItem,
        ChannelInterface $channel
    )
    {
        $orderItem->getOrder()->willReturn($order);
        $order->getChannel()->willReturn($channel);
        $channel->getCode()->willReturn('fr_FR');
        $channel->getName()->willReturn('default');

        $this
            ->shouldThrow(MissingChannelConfigurationException::class)
            ->during('calculate', [$orderItem, []]);
    }
}
