<?php

namespace spec\BitBag\SyliusBonusPointsPlugin\Checker\Eligibility;

use BitBag\SyliusBonusPointsPlugin\Checker\Eligibility\BonusPointsStrategyEligibilityCheckerInterface;
use BitBag\SyliusBonusPointsPlugin\Checker\Rule\BonusPointsStrategyRuleCheckerInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsStrategy;
use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsStrategyInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsStrategyRuleInterface;
use Doctrine\Common\Collections\ArrayCollection;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;
use Sylius\Component\Core\Model\OrderItemInterface;
use Sylius\Component\Registry\ServiceRegistryInterface;

class BonusPointsStrategyRulesEligibilityCheckerSpec extends ObjectBehavior
{
    function it_is_a_bonus_points_strategy_eligibility_checker()
    {
        $this->shouldImplement(BonusPointsStrategyEligibilityCheckerInterface::class);
    }

    function let(
        ServiceRegistryInterface $ruleRegistry,
        BonusPointsStrategyRuleCheckerInterface $checker
    )
    {
        $ruleRegistry->get('default')->willReturn($checker);
        $this->beConstructedWith($ruleRegistry);
    }

    function it_is_not_eligible_if_the_strategy_is_not_an_order_type_strategy(
        OrderItemInterface $orderItem,
        BonusPointsStrategyInterface $bonusPointsStrategy
    )
    {
        $bonusPointsStrategy->getType()->willReturn(Argument::any());
        $bonusPointsStrategy->hasRules()->shouldNotBeCalled();

        $this->isEligible($orderItem, $bonusPointsStrategy)->shouldReturn(false);
    }

    function it_is_not_eligible_if_the_strategy_has_no_rules(
        OrderItemInterface $orderItem,
        BonusPointsStrategyInterface $bonusPointsStrategy
    )
    {
        $bonusPointsStrategy->getType()->willReturn(BonusPointsStrategy::TYPE_ORDER);
        $bonusPointsStrategy->hasRules()->willReturn(false);
        $bonusPointsStrategy->getRules()->shouldNotBeCalled();

        $this->isEligible($orderItem, $bonusPointsStrategy)->shouldReturn(false);
    }

    function it_is_not_eligible_if_the_order_item_is_not_eligible_to_one_of_the_strategy_rules(
        OrderItemInterface $orderItem,
        BonusPointsStrategyInterface $bonusPointsStrategy,
        BonusPointsStrategyRuleInterface $firstRule,
        BonusPointsStrategyRuleInterface $secondRule,
        BonusPointsStrategyRuleCheckerInterface $checker
    )
    {
        $bonusPointsStrategy->getType()->willReturn(BonusPointsStrategy::TYPE_ORDER);
        $bonusPointsStrategy->hasRules()->willReturn(true);
        $bonusPointsStrategy->getRules()->willReturn(new ArrayCollection([$firstRule->getWrappedObject(), $secondRule->getWrappedObject()]));

        $firstRule->getType()->willReturn('default');
        $secondRule->getType()->willReturn('default');

        $firstRule->getConfiguration()->willReturn(['foo' => 'bar']);
        $secondRule->getConfiguration()->willReturn(['bar' => 'baz']);

        $checker->isEligible($orderItem, ['foo' => 'bar'])->willReturn(true);
        $checker->isEligible($orderItem, ['bar' => 'baz'])->willReturn(false);

        $this->isEligible($orderItem, $bonusPointsStrategy)->shouldReturn(false);
    }

    function it_is_eligible_if_the_order_item_is_eligible_to_all_the_strategy_rules(
        OrderItemInterface $orderItem,
        BonusPointsStrategyInterface $bonusPointsStrategy,
        BonusPointsStrategyRuleInterface $firstRule,
        BonusPointsStrategyRuleInterface $secondRule,
        BonusPointsStrategyRuleCheckerInterface $checker
    )
    {
        $bonusPointsStrategy->getType()->willReturn(BonusPointsStrategy::TYPE_ORDER);
        $bonusPointsStrategy->hasRules()->willReturn(true);
        $bonusPointsStrategy->getRules()->willReturn(new ArrayCollection([$firstRule->getWrappedObject(), $secondRule->getWrappedObject()]));

        $firstRule->getType()->willReturn('default');
        $secondRule->getType()->willReturn('default');

        $firstRule->getConfiguration()->willReturn(['foo' => 'bar']);
        $secondRule->getConfiguration()->willReturn(['bar' => 'baz']);

        $checker->isEligible($orderItem, ['foo' => 'bar'])->willReturn(true);
        $checker->isEligible($orderItem, ['bar' => 'baz'])->willReturn(true);

        $this->isEligible($orderItem, $bonusPointsStrategy)->shouldReturn(true);
    }
}
