<?php

namespace spec\BitBag\SyliusBonusPointsPlugin\Checker\Rule;

use BitBag\SyliusBonusPointsPlugin\Checker\Rule\BonusPointsStrategyRuleCheckerInterface;
use Doctrine\Common\Collections\ArrayCollection;
use InvalidArgumentException;
use PhpSpec\ObjectBehavior;
use Sylius\Component\Core\Model\OrderItemInterface;
use Sylius\Component\Core\Model\ProductInterface;
use Sylius\Component\Core\Model\TaxonInterface;
use Sylius\Component\Promotion\Exception\UnsupportedTypeException;
use Sylius\Component\Taxonomy\Repository\TaxonRepositoryInterface;

class HasTaxonRuleCheckerSpec extends ObjectBehavior
{
    function it_is_a_bonus_points_strategy_rule()
    {
        $this->shouldImplement(BonusPointsStrategyRuleCheckerInterface::class);
    }

    function let(TaxonRepositoryInterface $taxonRepository)
    {
        $this->beConstructedWith($taxonRepository);
    }

    function it_returns_true_if_the_product_has_at_least_one_of_the_taxons(
        OrderItemInterface $orderItem,
        TaxonRepositoryInterface $taxonRepository,
        TaxonInterface $tShirtsTaxon,
        TaxonInterface $jeansTaxons,
        ProductInterface $product
    )
    {
        $configuration['taxons'] = ['t-shirts', 'jeans'];
        $taxonRepository
            ->findBy(['code' => ['t-shirts', 'jeans']])
            ->willReturn(new ArrayCollection([$tShirtsTaxon->getWrappedObject(), $jeansTaxons->getWrappedObject()]));
        $orderItem->getProduct()->willReturn($product);
        $product->hasTaxon($tShirtsTaxon)->willReturn(false);
        $product->hasTaxon($jeansTaxons)->willReturn(true);

        $this->isEligible($orderItem, $configuration)->shouldReturn(true);
    }

    function it_returns_false_if_the_product_has_none_one_of_the_taxons(
        OrderItemInterface $orderItem,
        ProductInterface $product,
        TaxonInterface $tShirtsTaxon,
        TaxonInterface $jeansTaxons,
        TaxonRepositoryInterface $taxonRepository
    )
    {
        $orderItem->getProduct()->willReturn($product);
        $product->hasTaxon($tShirtsTaxon)->willReturn(false);
        $product->hasTaxon($jeansTaxons)->willReturn(false);

        $configuration['taxons'] = ['t-shirts', 'jeans'];
        $taxonRepository
            ->findBy(['code' => ['t-shirts', 'jeans']])
            ->willReturn(new ArrayCollection([$tShirtsTaxon->getWrappedObject(), $jeansTaxons->getWrappedObject()]));

        $this->isEligible($orderItem, $configuration)->shouldReturn(false);
    }

    function it_throws_an_unsupported_type_exception_if_the_order_item_is_not_a_product(
        OrderItemInterface $orderItem
    )
    {
        $orderItem->getProduct()->willReturn(null);

        $this
            ->shouldThrow(UnsupportedTypeException::class)
            ->during('isEligible', [$orderItem, []]);
    }

    function it_throws_an_invalid_argument_exception_if_the_configuration_has_no_taxons(
        OrderItemInterface $orderItem,
        ProductInterface $product
    )
    {
        $orderItem->getProduct()->willReturn($product);

        $this
            ->shouldThrow(InvalidArgumentException::class)
            ->during('isEligible', [$orderItem, []]);
    }
}
