<?php

namespace spec\BitBag\SyliusBonusPointsPlugin\Checker\Rule;

use BitBag\SyliusBonusPointsPlugin\Checker\Rule\BonusPointsStrategyRuleCheckerInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsStrategyInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\CustomerBonusPointsStrategyInterface;
use InvalidArgumentException;
use PhpSpec\ObjectBehavior;
use Sylius\Component\Core\Model\CustomerInterface;
use Sylius\Component\Core\Model\OrderInterface;
use Sylius\Component\Core\Model\OrderItemInterface;
use Sylius\Component\Core\OrderPaymentStates;
use Sylius\Component\Core\Repository\OrderRepositoryInterface;
use Sylius\Component\Resource\Repository\RepositoryInterface;

class NumberOfOrdersRuleCheckerSpec extends ObjectBehavior
{
    function it_is_a_bonus_points_strategy_rule()
    {
        $this->shouldImplement(BonusPointsStrategyRuleCheckerInterface::class);
    }

    function let(
        OrderRepositoryInterface $orderRepository
    )
    {
        $this->beConstructedWith($orderRepository);
    }

    function it_returns_true_if_there_is_no_customer_for_the_order(
        OrderItemInterface $orderItem,
        OrderInterface $order
    )
    {
        $orderItem->getOrder()->willReturn($order);
        $order->getCustomer()->willReturn(null);

        $this
            ->isEligible($orderItem, ['number' => ''])
            ->shouldReturn(true);
    }

    function it_returns_false_if_the_number_of_paid_orders_including_the_subject_is_greater_than_the_configured_number(
        OrderItemInterface $orderItem,
        OrderInterface $order,
        CustomerInterface $customer,
        OrderRepositoryInterface $orderRepository,
        OrderInterface $orderOne,
        OrderInterface $orderTwo
    )
    {
        $orderItem->getOrder()->willReturn($order);
        $order->getCustomer()->willReturn($customer);

        $orderRepository
            ->findBy([
                'customer' => $customer,
                'paymentState' => OrderPaymentStates::STATE_PAID
            ])
            ->willReturn([
                $orderOne,
                $orderTwo
            ])
            ->shouldBeCalledOnce();

        $this
            ->isEligible($orderItem, ['number' => '2'])
            ->shouldReturn(true);
    }

    function it_returns_false_if_the_number_of_paid_orders_including_the_subject_is_equal_to_the_configured_number(
        OrderItemInterface $orderItem,
        OrderInterface $order,
        CustomerInterface $customer,
        OrderRepositoryInterface $orderRepository,
        OrderInterface $orderOne
    )
    {
        $orderItem->getOrder()->willReturn($order);
        $order->getCustomer()->willReturn($customer);

        $orderRepository
            ->findBy([
                'customer' => $customer,
                'paymentState' => OrderPaymentStates::STATE_PAID
            ])
            ->willReturn([
                $orderOne
            ])
            ->shouldBeCalledOnce();

        $this
            ->isEligible($orderItem, ['number' => '2'])
            ->shouldReturn(false);
    }

    function it_returns_false_if_the_number_of_paid_orders_including_the_subject_is_lower_than_the_configured_number(
        OrderItemInterface $orderItem,
        OrderInterface $order,
        CustomerInterface $customer,
        OrderRepositoryInterface $orderRepository,
        OrderInterface $orderOne
    )
    {
        $orderItem->getOrder()->willReturn($order);
        $order->getCustomer()->willReturn($customer);

        $orderRepository
            ->findBy([
                'customer' => $customer,
                'paymentState' => OrderPaymentStates::STATE_PAID
            ])
            ->willReturn([
                $orderOne
            ])
            ->shouldBeCalledOnce();

        $this
            ->isEligible($orderItem, ['number' => '3'])
            ->shouldReturn(false);
    }

    function it_throws_an_invalid_argument_exception_if_the_configuration_has_no_number(
        OrderItemInterface $orderItem
    )
    {
        $this
            ->shouldThrow(InvalidArgumentException::class)
            ->during('isEligible', [$orderItem, []]);
    }

    function it_throws_an_invalid_argument_exception_if_the_order_item_is_not_attached_to_an_order(
        OrderItemInterface $orderItem
    )
    {
        $orderItem->getOrder()->willReturn(null);

        $this
            ->shouldThrow(InvalidArgumentException::class)
            ->during('isEligible', [$orderItem, ['strategyCode' => '']]);
    }
}
