<?php

namespace spec\BitBag\SyliusBonusPointsPlugin\Context;

use BitBag\SyliusBonusPointsPlugin\Context\CustomerBonusPointsContext;
use BitBag\SyliusBonusPointsPlugin\Context\CustomerBonusPointsContextInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\CustomerBonusPointsInterface;
use PhpSpec\ObjectBehavior;
use Sylius\Component\Core\Model\CustomerInterface;
use Sylius\Component\Customer\Context\CustomerContextInterface;
use Sylius\Component\Resource\Factory\FactoryInterface;
use Sylius\Component\Resource\Repository\RepositoryInterface;

class CustomerBonusPointsContextSpec extends ObjectBehavior
{
    function it_is_a_customer_bonus_points_context()
    {
        $this->shouldImplement(CustomerBonusPointsContextInterface::class);
    }

    function let(
        CustomerContextInterface $customerContext,
        RepositoryInterface $customerBonusPointsRepository,
        FactoryInterface $customerBonusPointsFactory
    )
    {
        $this->beConstructedWith($customerContext, $customerBonusPointsRepository, $customerBonusPointsFactory);
    }

    function it_returns_null_if_there_is_not_customer_in_the_context(
        CustomerContextInterface $customerContext
    )
    {
        $customerContext->getCustomer()->willReturn(null);

        $this->getCustomerBonusPoints()->shouldReturn(null);
    }

    function it_returns_the_existing_bonus_points_for_the_customer(
        CustomerContextInterface $customerContext,
        RepositoryInterface $customerBonusPointsRepository,
        CustomerInterface $customer,
        CustomerBonusPointsInterface $customerBonusPoints
    )
    {
        $customerContext->getCustomer()->willReturn($customer);
        $customerBonusPointsRepository->findOneBy(['customer' => $customer])->willReturn($customerBonusPoints);
        $this->getCustomerBonusPoints()->shouldReturn($customerBonusPoints);
    }

    function it_create_a_bonus_points_entry_if_there_is_none_for_the_customer(
        CustomerContextInterface $customerContext,
        RepositoryInterface $customerBonusPointsRepository,
        FactoryInterface $customerBonusPointsFactory,
        CustomerInterface $customer,
        CustomerBonusPointsInterface $customerBonusPoints
    )
    {
        $customerContext->getCustomer()->willReturn($customer);
        $customerBonusPointsRepository->findOneBy(['customer' => $customer])->willReturn(null);

        $customerBonusPointsFactory->createNew()->willReturn($customerBonusPoints)->shouldBeCalledOnce();
        $customerBonusPointsRepository->add($customerBonusPoints)->shouldBeCalledOnce();

        $this->getCustomerBonusPoints()->shouldReturn($customerBonusPoints);
    }
}
