<?php

namespace spec\BitBag\SyliusBonusPointsPlugin\OrderProcessing;

use BitBag\SyliusBonusPointsPlugin\Entity\AdjustmentInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsAwareInterface;
use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsInterface;
use BitBag\SyliusBonusPointsPlugin\OrderProcessing\OrderBonusPointsProcessor;
use Mockery\Matcher\Any;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;
use Sylius\Component\Core\Model\OrderInterface;
use Sylius\Component\Order\Factory\AdjustmentFactoryInterface;
use Sylius\Component\Order\Processor\OrderProcessorInterface;
use Sylius\Component\Resource\Repository\RepositoryInterface;

class OrderBonusPointsProcessorSpec extends ObjectBehavior
{
    function it_is_an_order_processor()
    {
        $this->shouldImplement(OrderProcessorInterface::class);
    }

    function let(
        RepositoryInterface $bonusPointsRepository,
        AdjustmentFactoryInterface $adjustmentFactory
    ) {
        $this->beConstructedWith($bonusPointsRepository, $adjustmentFactory);
    }

    function it_throws_an_invalid_argument_exception_if_the_order_is_not_bonus_points_aware(
        OrderInterface $order
    )
    {
        $this
            ->shouldThrow(\InvalidArgumentException::class)
            ->during('process', [$order]);
    }

    function it_does_nothing_if_there_is_no_used_bonus_points_for_the_order(
        RepositoryInterface $bonusPointsRepository,
        OrderInterface $order
    )
    {
        $order->implement(BonusPointsAwareInterface::class);
        $bonusPointsRepository->findOneBy([
            'order' => $order,
            'isUsed' => true
        ])->willReturn(null);

        $order->removeAdjustment(Argument::any())->shouldNotBeCalled();
        $order->addAdjustment(Argument::any())->shouldNotBeCalled();

        $this->process($order);
    }

    function it_replaces_existing_bonus_points_adjustments(
        RepositoryInterface $bonusPointsRepository,
        AdjustmentFactoryInterface $adjustmentFactory,
        OrderInterface $order,
        BonusPointsInterface $bonusPoints,
        AdjustmentInterface $adjustment
    )
    {
        $order->implement(BonusPointsAwareInterface::class);
        $bonusPointsRepository->findOneBy([
            'order' => $order,
            'isUsed' => true
        ])->willReturn($bonusPoints);

        $order->removeAdjustments(AdjustmentInterface::ORDER_BONUS_POINTS_ADJUSTMENT)->shouldBeCalledOnce();

        $bonusPoints->getPoints()->willReturn(100);

        $adjustmentFactory->createWithData(
            AdjustmentInterface::ORDER_BONUS_POINTS_ADJUSTMENT,
            AdjustmentInterface::ORDER_BONUS_POINTS_ADJUSTMENT,
            -10000
        )->willReturn($adjustment);

        $adjustment->setOriginCode(AdjustmentInterface::ORDER_BONUS_POINTS_ADJUSTMENT)->shouldBeCalledOnce();
        $adjustment->setAdjustable($order)->shouldBeCalledOnce();

        $order->addAdjustment($adjustment)->shouldBeCalledOnce();

        $this->process($order);
    }
}
