<?php

declare(strict_types=1);

namespace BitBag\SyliusBonusPointsPlugin\Calculator;

use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsStrategy;
use Sylius\Component\Channel\Context\ChannelContextInterface;
use Sylius\Component\Core\Exception\MissingChannelConfigurationException;
use Sylius\Component\Core\Model\ChannelInterface;
use Sylius\Component\Core\Model\OrderInterface;

final class FixedAmountCalculator implements BonusPointsStrategyCalculatorInterface
{
    private $channelContext;

    public function __construct(ChannelContextInterface $channelContext)
    {
        $this->channelContext = $channelContext;
    }

    public function calculate($subject, array $configuration, int $amountToDeduct = 0): int
    {
        /** @var ChannelInterface $channel */
        $channel = $subject instanceof OrderInterface ? $subject->getChannel() : $this->channelContext->getChannel();

        $channelCode = $channel->getCode();

        if (!isset($configuration[$channelCode]['amount'])) {
            throw new MissingChannelConfigurationException(sprintf(
                'Channel %s has no amount defined for calculator %s',
                $channel->getName(),
                $this->getType()
            ));
        }

        return $configuration[$channelCode]['amount'];
    }

    public function isPerOrderItem(): bool
    {
        return false;
    }

    public function getType(): string
    {
        return 'fixed_amount';
    }

    public function getCompatibleStrategyTypes(): array
    {
        return [
            BonusPointsStrategy::TYPE_ORDER,
            BonusPointsStrategy::TYPE_EVENT,
        ];
    }
}
