<?php

declare(strict_types=1);

namespace BitBag\SyliusBonusPointsPlugin\Calculator;

use BitBag\SyliusBonusPointsPlugin\Entity\BonusPointsStrategy;
use Sylius\Component\Core\Exception\MissingChannelConfigurationException;
use Sylius\Component\Core\Model\ChannelInterface;
use Sylius\Component\Core\Model\OrderInterface;
use Sylius\Component\Core\Model\OrderItemInterface;
use Webmozart\Assert\Assert;

final class PerOrderItemPercentageCalculator implements BonusPointsStrategyCalculatorInterface
{
    public function calculate($subject, array $configuration, int $amountToDeduct = 0): int
    {
        /** @var OrderItemInterface $subject */
        Assert::isInstanceOf($subject, OrderItemInterface::class);

        /** @var OrderInterface $order */
        $order = $subject->getOrder();

        /** @var ChannelInterface $channel */
        $channel = $order->getChannel();

        $channelCode = $channel->getCode();

        if (!isset($configuration[$channelCode]['percentToCalculatePoints'])) {
            throw new MissingChannelConfigurationException(sprintf(
                'Channel %s has no amount defined for calculator %s',
                $channel->getName(),
                $this->getType()
            ));
        }

        $total = $subject->getTotal();

        if ($amountToDeduct < 0) {
            $total += $amountToDeduct;
        }

        $result = (int) floor($total * $configuration[$channelCode]['percentToCalculatePoints']);

        return ($result === 0) ? 0 : (int) floor(($total * $configuration[$channelCode]['percentToCalculatePoints']) / 100);
    }

    public function isPerOrderItem(): bool
    {
        return true;
    }

    public function getType(): string
    {
        return 'per_order_item_percentage';
    }

    public function getCompatibleStrategyTypes(): array
    {
        return [
            BonusPointsStrategy::TYPE_ORDER,
        ];
    }
}
